@@ -26,12 +26,15 @@ class UnknownError(RpcException):
 class NotAuthorizedError(RpcException):
     pass
 
+class PermissionDeniedError(RpcException):
+    pass
+
 class RpcExchangerRpcHandler2:
     def __init__(self, exchanger: RpcExchanger,  provider: str) -> None:
         self._exchanger = exchanger
         self._provider = provider
 
-    def __getattr__(self, service: str):
+    def __getattr__(self, service: str) -> Callable[..., Awaitable[Any]]:
         async def func(**data):
             result = await self._exchanger.rpc_request(self._provider+"/"+service, data)
             log.debug(f"{result=}")
@@ -55,7 +58,6 @@ class RpcExchanger:
     _jobid: str
     def __init__(self) -> None:
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        sock.setblocking(False)
         sock.bind(("0.0.0.0", 0))
         self._sock = sock
         self._lock = asyncio.Lock()
@@ -63,9 +65,24 @@ class RpcExchanger:
         self._redis = redis.Redis(host="localhost")
         self.rpc = RpcExchangerRpcHandler(self)
 
+    def __enter__(self) -> RpcExchanger:
+        # you should not use this, use async version instead
+        sock = self._sock
+        sock.connect(("127.0.0.1", 2474))
+        sock.send(b'{"type": "handshake","role": "client"}')
+        self._jobid = json.loads(sock.recv(65536).decode())["initial_jobid"]
+        sock.setblocking(False)
+        return self
+
+    def __exit__(self, *args) -> None:
+        # redis close
+        self._sock.shutdown(socket.SHUT_RDWR)
+        self._sock.close()
+
     async def __aenter__(self) -> RpcExchanger:
         loop = asyncio.get_event_loop()
         sock = self._sock
+        sock.setblocking(False)
         await loop.sock_connect(sock, ("127.0.0.1", 2474))
         await loop.sock_sendall(sock, b'{"type": "handshake","role": "client"}')
         self._jobid = json.loads((await loop.sock_recv(sock, 65536)).decode())["initial_jobid"]
@@ -150,6 +167,8 @@ class RpcExchangerClient:
             raise ChatNotJoinedError()
         if self._username is None:
             raise NotAuthorizedError()
+        if not await self._rpc.chat.check_send(chat_id=chat, user_id=self._uid):
+            raise PermissionDeniedError()
         log.debug(f"{self._chat_list=}")
         await self._exchanger.send_msg(self._username, msg, chat)    
     
@@ -240,12 +259,28 @@ class RpcExchangerClient:
         self.check_authorized()
         if self._uid == kicked_user_id:
             return False
+        if chat_id not in self._chat_list:
+            raise ChatNotJoinedError()
         return await self._rpc.chat.kick(chat_id=chat_id, user_id=self._uid, kicked_user_id=kicked_user_id)
 
     async def chat_rename(self, chat_id: int, new_name: str) -> bool:
         self.check_authorized()
+        if chat_id not in self._chat_list:
+            raise ChatNotJoinedError()
         return await self._rpc.chat.rename(chat_id=chat_id, user_id=self._uid, new_name=new_name)
 
+    async def chat_invite(self, chat_id: int, invited_user_id: int) -> bool:
+        self.check_authorized()
+        if chat_id not in self._chat_list:
+            raise ChatNotJoinedError()
+        return await self._rpc.chat.invite(chat_id=chat_id, user_id=self._uid, invited_user_id=invited_user_id)
+
+    async def chat_modify_user_permission(self, chat_id: int, modified_user_id: int, name: str, value: bool) -> bool:
+        self.check_authorized()
+        if chat_id not in self._chat_list:
+            raise ChatNotJoinedError()
+        return await self._rpc.chat.kick(chat_id=chat_id, user_id=self._uid, modified_user_id=modified_user_id, name=name, value=value)
+
     def __aiter__(self) -> RpcExchangerClient:
         return self
 
@@ -261,4 +296,4 @@ class RpcExchangerClient:
         return None
 
 
-__all__ = ["RpcExchanger", "RpcExchangerClient", "ChatAlreadyJoinedError", "ChatNotJoinedError", "UnknownError", "NotAuthorizedError"]
+__all__ = ["RpcExchanger", "RpcExchangerClient", "ChatAlreadyJoinedError", "ChatNotJoinedError", "UnknownError", "NotAuthorizedError", "PermissionDeniedError"]
